/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "quakedef.h"
#include "particles.h"
#include "resource.h"

ptypedef_t p_typedefs[MAX_PARTICLE_TYPES] = {
	{{0, 0, 0}, {0, 0, 0}, NULL, 0},		// pt_static
	{{0, 0, 0}, {0, 0, -1}, NULL, 0},		// pt_grav
	{{0, 0, 0}, {0, 0, -0.5}, NULL, 0},		// pt_slowgrav
	{{0, 0, 0}, {0, 0, 1}, ramp3, 5},		// pt_fire
	{{4, 4, 4}, {0, 0, -1}, ramp1, 10},		// pt_explode
	{{-1, -1, -1}, {0, 0, -1}, ramp2, 15},	// pt_explode2
	{{4, 4, 4}, {0, 0, -1}, NULL, 0},		// pt_blob
	{{-4, -4, 0}, {0, 0, -1}, NULL, 0},		// pt_blob2
};


void R_DrawParticles (void)
{
	emitter_t *e;
	static const float r_particleoffsets[4][2] = {{-1, -1}, {-1, 1}, {1, 1}, {1, -1}};
	qboolean pBegun = false;

	for (e = active_emitters; e; e = e->next)
	{
		particle_t *p;

		// ???? should never happen
		if (!e->particles) continue;

		// bbcull the emitter
		if (R_CullBox (e->mins, e->maxs)) continue;

		for (p = e->particles; p; p = p->next)
		{
			// particle was removed
			if (p->die < cl.time) continue;

			// don't draw full-alpha particles
			if ((p->color & 255) == 0xff) continue;

			if (!pBegun)
			{
				// set state
				GL_BlendState (BS_ALPHABLEND);
				GL_DepthState (DS_TESTLESSEQUAL);
				GL_BindPrograms (r_particlevp, r_particlefp);

				// set shader params
				glProgramLocalParameter4fvARB (GL_VERTEX_PROGRAM_ARB, 1, r_refdef.vieworigin);
				glProgramLocalParameter4fvARB (GL_VERTEX_PROGRAM_ARB, 2, vpn);
				glProgramLocalParameter4fvARB (GL_VERTEX_PROGRAM_ARB, 3, vup);
				glProgramLocalParameter4fvARB (GL_VERTEX_PROGRAM_ARB, 4, vright);
				glProgramLocalParameter4fARB (GL_VERTEX_PROGRAM_ARB, 5, 0.666f, 0.002f, 1.0f, 0.0f);

				glBegin (GL_QUADS);

				pBegun = true;
			}

			glVertexAttrib3fv (2, p->move);
			glVertexAttrib4Nubv (1, (byte *) &d_8to24table_rgba[p->color & 255]);

			glVertexAttrib2fv (0, r_particleoffsets[0]);
			glVertexAttrib2fv (0, r_particleoffsets[1]);
			glVertexAttrib2fv (0, r_particleoffsets[2]);
			glVertexAttrib2fv (0, r_particleoffsets[3]);
		}
	}

	if (pBegun) glEnd ();
}

