/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// r_surf.c: surface-related refresh code

#include "quakedef.h"
#include "resource.h"


int Sys_LoadResourceData (int resourceid, void **resbuf)
{
	// per MSDN, UnlockResource is obsolete and does nothing any more.  There is
	// no way to free the memory used by a resource after you're finished with it.
	// If you ask me this is kinda fucked, but what do I know?  We'll just leak it.
	if (resbuf)
	{
		HRSRC hResInfo = FindResource (NULL, MAKEINTRESOURCE (resourceid), RT_RCDATA);

		if (hResInfo)
		{
			HGLOBAL hResData = LoadResource (NULL, hResInfo);

			if (hResData)
			{
				resbuf[0] = (byte *) LockResource (hResData);
				return SizeofResource (NULL, hResInfo);
			}
		}
	}

	return 0;
}


GLuint GL_CreateProgram (GLenum mode, int resourceID)
{
	GLuint progid = 0;
	char *progstr = NULL;

	int len = Sys_LoadResourceData (resourceID, (void **) &progstr);

	glGenProgramsARB (1, &progid);
	glBindProgramARB (mode, progid);
	glProgramStringARB (mode, GL_PROGRAM_FORMAT_ASCII_ARB, len, progstr);
	glBindProgramARB (mode, 0);

	return progid;
}


GLuint r_depthpasssurfvp = 0;
GLuint r_depthpassmeshvp = 0;
GLuint r_depthpassfp = 0;

GLuint r_drawtexturedvp = 0;
GLuint r_drawtexturedfp = 0;

GLuint r_drawcolouredvp = 0;
GLuint r_drawcolouredfp = 0;

GLuint r_fullbrightsurfvp = 0;
GLuint r_fullbrightmeshvp = 0;
GLuint r_fullbrightfp = 0;

GLuint r_lightsurfvp = 0;
GLuint r_lightmeshvp = 0;

GLuint r_pointlightfp = 0;
GLuint r_spotlightfp = 0;

GLuint r_skysurfvp = 0;
GLuint r_skysurffp = 0;

GLuint r_warpsurfvp = 0;
GLuint r_warpsurffp = 0;

GLuint r_particlevp = 0;
GLuint r_particlefp = 0;

void GL_LoadShaders (void)
{
	r_depthpasssurfvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_DEPTHPASSSURFVP);
	r_depthpassmeshvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_DEPTHPASSMESHVP);
	r_depthpassfp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_DEPTHPASSFP);

	r_drawtexturedvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_DRAWTEXTUREDVP);
	r_drawtexturedfp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_DRAWTEXTUREDFP);

	r_drawcolouredvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_DRAWCOLOUREDVP);
	r_drawcolouredfp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_DRAWCOLOUREDFP);

	r_lightsurfvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_LIGHTSURFVP);
	r_lightmeshvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_LIGHTMESHVP);

	r_fullbrightsurfvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_FULLBRIGHTSURFVP);
	r_fullbrightmeshvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_FULLBRIGHTMESHVP);
	r_fullbrightfp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_FULLBRIGHTFP);

	r_pointlightfp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_POINTLIGHTFP);
	r_spotlightfp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_SPOTLIGHTFP);

	r_skysurfvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_SKYSURFVP);
	r_skysurffp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_SKYSURFFP);

	r_warpsurfvp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_WARPSURFVP);
	r_warpsurffp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_WARPSURFFP);

	r_particlevp = GL_CreateProgram (GL_VERTEX_PROGRAM_ARB, IDR_PARTICLEVP);
	r_particlefp = GL_CreateProgram (GL_FRAGMENT_PROGRAM_ARB, IDR_PARTICLEFP);

	glEnable (GL_VERTEX_PROGRAM_ARB);
	glEnable (GL_FRAGMENT_PROGRAM_ARB);
}


static GLuint currentvp = 0;
static GLuint currentfp = 0;

void GL_BindPrograms (GLuint vp, GLuint fp)
{
	if (currentvp != vp)
	{
		glBindProgramARB (GL_VERTEX_PROGRAM_ARB, vp);
		currentvp = vp;
	}

	if (currentfp != fp)
	{
		glBindProgramARB (GL_FRAGMENT_PROGRAM_ARB, fp);
		currentfp = fp;
	}
}


GLuint GL_LoadGLSL (int resourceID)
{
	GLuint progid = glCreateProgram ();
	GLuint vs = glCreateShader (GL_VERTEX_SHADER);
	GLuint fs = glCreateShader (GL_FRAGMENT_SHADER);
	char *progstr = NULL;

	int len = Sys_LoadResourceData (resourceID, (void **) &progstr);

	glShaderSource (vs, 1, &progstr, &len);
	glShaderSource (fs, 1, &progstr, &len);

	glCompileShader (vs);
	glCompileShader (fs);

	glAttachShader (progid, vs);
	glAttachShader (progid, fs);

	glLinkProgram (progid);

	return progid;
}

